@@ -4,7 +4,7 @@ package DBIx::SearchBuilder;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = "1.43";
+$VERSION = "1.45";
 
 use Clone qw();
 
@@ -649,6 +649,14 @@ Unless ALIAS is set, the join criterias will be taken from EXT_LINKFIELD
 and INT_LINKFIELD and added to the criterias.  If ALIAS is set, new
 criterias about the foreign table will be added.
 
+=item LEFTJOIN
+
+To apply the Limit inside the ON clause of a previously created left
+join, pass this option along with the alias returned from creating
+the left join. ( This is similar to using the EXPRESSION option when
+creating a left join but this allows you to refer to the join alias
+in the expression. )
+
 =item FIELD
 
 Column to be checked against.
@@ -690,7 +698,9 @@ doesn't surround the string in % signs as LIKE does.
 
 =item ENTRYAGGREGATOR 
 
-Can be AND or OR (or anything else valid to aggregate two clauses in SQL)
+Can be C<AND> or C<OR> (or anything else valid to aggregate two clauses in SQL).
+Special value is C<none> which means that no entry aggregator should be used.
+The default value is C<OR>.
 
 =item CASESENSITIVE
 
@@ -699,7 +709,7 @@ this search case sensitive
 
 =back
 
-=cut 
+=cut
 
 sub Limit {
     my $self = shift;
@@ -740,11 +750,11 @@ sub Limit {
         }
         elsif ( $args{'OPERATOR'} =~ /STARTSWITH/i ) {
             $args{'VALUE'}    = $args{'VALUE'} . "%";
-            $args{'OPERATOR'} =~ s/STARTSWITH/LIKE/;
+            $args{'OPERATOR'} =~ s/STARTSWITH/LIKE/i;
         }
         elsif ( $args{'OPERATOR'} =~ /ENDSWITH/i ) {
             $args{'VALUE'}    = "%" . $args{'VALUE'};
-            $args{'OPERATOR'} =~ s/ENDSWITH/LIKE/;
+            $args{'OPERATOR'} =~ s/ENDSWITH/LIKE/i;
         } 
 	
 	$args{'OPERATOR'} =~ s/MATCHES/LIKE/i;  # MATCHES becomes LIKE, with no % stuff
@@ -833,8 +843,6 @@ sub _GenericRestriction {
                  QUOTEVALUE     => undef,
                  @_ );
 
-    my ( $Clause, $QualifiedField );
-
     #TODO: $args{'VALUE'} should take an array of values and generate
     # the proper where clause.
 
@@ -874,32 +882,26 @@ sub _GenericRestriction {
     # Set this to the name of the field and the alias, unless we've been
     # handed a subclause name
 
-    $QualifiedField = $args{'ALIAS'} . "." . $args{'FIELD'};
-
-    if ( $args{'SUBCLAUSE'} ) {
-        $Clause = $args{'SUBCLAUSE'};
-    }
-    else {
-        $Clause = $QualifiedField;
-    }
+    my $QualifiedField = $args{'ALIAS'} . "." . $args{'FIELD'};
+    my $ClauseId = $args{'SUBCLAUSE'} || $QualifiedField;
 
     print STDERR "$self->_GenericRestriction QualifiedField=$QualifiedField\n"
       if ( $self->DEBUG );
 
-    my ($restriction);
 
     # If we're trying to get a leftjoin restriction, lets set
     # $restriction to point htere. otherwise, lets construct normally
 
+    my ($restriction);
     if ( $args{'LEFTJOIN'} ) {
-        if ($args{'ENTRYAGGREGATOR'} ) {
+        if ( $args{'ENTRYAGGREGATOR'} ) {
             $self->{'left_joins'}{ $args{'LEFTJOIN'} }{'entry_aggregator'} = 
                 $args{'ENTRYAGGREGATOR'};
         }
-        $restriction = \$self->{'left_joins'}{ $args{'LEFTJOIN'} }{'criteria'}{"$Clause"};
+        $restriction = \$self->{'left_joins'}{ $args{'LEFTJOIN'} }{'criteria'}{ $ClauseId };
     }
     else {
-        $restriction = \$self->{'restrictions'}{"$Clause"};
+        $restriction = \$self->{'restrictions'}{ $ClauseId };
     }
 
     # If it's a new value or we're overwriting this sort of restriction,
@@ -918,21 +920,15 @@ sub _GenericRestriction {
 
     # Juju because this should come _AFTER_ the EA
     my $prefix = "";
-    if ( $self->{_open_parens}{$Clause} ) {
-        $prefix = " ( " x $self->{_open_parens}{$Clause};
-        delete $self->{_open_parens}{$Clause};
+    if ( $self->{_open_parens}{ $ClauseId } ) {
+        $prefix = " ( " x delete $self->{_open_parens}{ $ClauseId };
     }
 
-    if ( (     ( exists $args{'ENTRYAGGREGATOR'} )
-           and ( $args{'ENTRYAGGREGATOR'} || "" ) eq 'none' )
-         or ( !$$restriction )
-      ) {
-
+    if ( lc( $args{'ENTRYAGGREGATOR'} || "" ) eq 'none' || !$$restriction ) {
         $$restriction = $prefix . $clause;
-
     }
     else {
-        $$restriction .= $args{'ENTRYAGGREGATOR'} . $prefix . $clause;
+        $$restriction .= " ". $args{'ENTRYAGGREGATOR'} . " " . $prefix . $clause;
     }
 
     return ( $args{'ALIAS'} );
@@ -942,13 +938,13 @@ sub _GenericRestriction {
 
 sub _OpenParen {
     my ( $self, $clause ) = @_;
-    $self->{_open_parens}{$clause}++;
+    $self->{_open_parens}{ $clause }++;
 }
 
 # Immediate Action
 sub _CloseParen {
     my ( $self, $clause ) = @_;
-    my $restriction = \$self->{'restrictions'}{"$clause"};
+    my $restriction = \$self->{'restrictions'}{ $clause };
     if ( !$$restriction ) {
         $$restriction = " ) ";
     }
@@ -963,7 +959,7 @@ sub _AddSubClause {
     my $clauseid  = shift;
     my $subclause = shift;
 
-    $self->{'subclauses'}{"$clauseid"} = $subclause;
+    $self->{'subclauses'}{ $clauseid } = $subclause;
 
 }
 
@@ -971,7 +967,6 @@ sub _AddSubClause {
 
 sub _WhereClause {
     my $self = shift;
-    my ( $subclause, $where_clause );
 
     #Go through all the generic restrictions and build up the "generic_restrictions" subclause
     # That's the only one that SearchBuilder builds itself.
@@ -980,21 +975,15 @@ sub _WhereClause {
 
     #Go through all restriction types. Build the where clause from the
     #Various subclauses.
-    foreach $subclause ( keys %{ $self->{'subclauses'} } ) {
-        # Now, build up the where clause
-        if ( defined($where_clause) ) {
-            $where_clause .= " AND ";
-        }
-
-        warn "$self $subclause doesn't exist"
-          if ( !defined $self->{'subclauses'}{"$subclause"} );
-        $where_clause .= $self->{'subclauses'}{"$subclause"};
+    my $where_clause = '';
+    foreach my $subclause ( grep $_, values %{ $self->{'subclauses'} } ) {
+        $where_clause .= " AND " if $where_clause;
+        $where_clause .= $subclause;
     }
 
-    $where_clause = " WHERE " . $where_clause if ( $where_clause ne '' );
+    $where_clause = " WHERE " . $where_clause if $where_clause;
 
     return ($where_clause);
-
 }
 
 
@@ -1003,18 +992,16 @@ sub _WhereClause {
 
 sub _CompileGenericRestrictions {
     my $self = shift;
-    my ($restriction);
 
-    delete $self->{'subclauses'}{'generic_restrictions'};
+    $self->{'subclauses'}{'generic_restrictions'} = '';
 
+    my $result = '';
     #Go through all the restrictions of this type. Buld up the generic subclause
-    foreach $restriction ( sort keys %{ $self->{'restrictions'} } ) {
-        if ( defined $self->{'subclauses'}{'generic_restrictions'} ) {
-            $self->{'subclauses'}{'generic_restrictions'} .= " AND ";
-        }
-        $self->{'subclauses'}{'generic_restrictions'} .=
-          "(" . $self->{'restrictions'}{"$restriction"} . ")";
+    foreach my $restriction ( sort keys %{ $self->{'restrictions'} } ) {
+        $result .= " AND " if $result;
+        $result .= "(" . $self->{'restrictions'}{ $restriction } . ")";
     }
+    return ($self->{'subclauses'}{'generic_restrictions'} = $result);
 }
 
 
@@ -1073,7 +1060,7 @@ sub _OrderClause {
 			ORDER => 'ASC',
 			%$row
 		      );
-        if ($rowhash{'ORDER'} =~ /^des/i) {
+        if ($rowhash{'ORDER'} && $rowhash{'ORDER'} =~ /^des/i) {
 	    $rowhash{'ORDER'} = "DESC";
         }
         else {
@@ -1640,7 +1627,7 @@ is installed, using a temporary file as the database.  For example:
 
 =head1 AUTHOR
 
-Copyright (c) 2001-2005 Jesse Vincent, jesse@fsck.com.
+Copyright (c) 2001-2006 Jesse Vincent, jesse@bestpractical.com.
 
 All rights reserved.
 
