@@ -1,8 +1,7 @@
 #!/usr/bin/perl -w
 
 use strict;
-use File::Temp qw/ tempdir /;
-use File::Spec;
+
 =head1 VARIABLES
 
 =head2 @SupportedDrivers
@@ -93,15 +92,13 @@ sub connect_handle_with_driver
 	goto &$call;
 }
 
-sub connect_sqlite {
-
-    my $dir = tempdir(CLEANUP => 1);
-    
-    my $handle = shift;
-    return $handle->Connect(
-        Driver   => 'SQLite',
-        Database => File::Spec->catfile($dir => "db.sqlite")
-    );
+sub connect_sqlite
+{
+	my $handle = shift;
+	return $handle->Connect(
+		Driver => 'SQLite',
+		Database => File::Spec->catfile(File::Spec->tmpdir(), "sb-test.$$")
+	);
 }
 
 sub connect_mysql
@@ -126,19 +123,6 @@ sub connect_pg
 	);
 }
 
-sub connect_oracle
-{
-	my $handle = shift;
-	return $handle->Connect(
-		Driver   => 'Oracle',
-		Database => $ENV{'SB_TEST_ORACLE'},
-		Host     => $ENV{'SB_TEST_ORACLE_HOST'},
-		SID      => $ENV{'SB_TEST_ORACLE_SID'},
-		User     => $ENV{'SB_TEST_ORACLE_USER'} || 'test',
-		Password => $ENV{'SB_TEST_ORACLE_PASS'} || 'test',
-    );
-}
-
 =head2 should_test
 
 Checks environment for C<SB_TEST_*> variables.
